//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***


#include <sys.h>

EXTERN void __tmr_fix();


#define HWREG(base,off) ( (volatile unsigned int *) (((int)base+(int)off)))

// ROMLIB CHECKED
int __cpu_getFCLK()
{
int CLKSLOW=*HWREG(CLK_REGS,0x10);
int FCLK=12000000;

if(CLKSLOW&0x10) {	// slow mode
	if(CLKSLOW&7) FCLK/= (CLKSLOW&7)<<1;
	}
else {				// fast mode
	int PLLCON=*HWREG(CLK_REGS,0x4);
	int m=((PLLCON>>12)&0xff)+8,p=((PLLCON>>4)&0x1f)+2,s=PLLCON&3;
	
	FCLK=((unsigned)m*12000000)/(p*(1<<s));
}
return FCLK;
}


int __cpu_getHCLK()
{
int FCLK=__cpu_getFCLK();
int CLKDIVN=*HWREG(CLK_REGS,0x14);
if(CLKDIVN&2) FCLK>>=1;
return FCLK;
}	

int __cpu_getPCLK()
{
int FCLK=__cpu_getFCLK();
int CLKDIVN=*HWREG(CLK_REGS,0x14);
if(CLKDIVN&2) FCLK>>=1;
if(CLKDIVN&1) FCLK>>=1;
return FCLK;
}	

int cpu_getspeed()
{
	return __cpu_getFCLK();
}

// DECLARATION MOVED TO HPSYS.H
/*
#define CLK_1MHZ 0x16000000
#define CLK_6MHZ 0x11000000
#define CLK_12MHZ 0x10000000
#define CLK_48MHZ 0x78023
#define CLK_75MHZ 0x43012
#define CLK_120MHZ 0x5c080
#define CLK_152MHZ 0x48011
#define CLK_192MHZ 0x58011
*/


int __cpu_setspeed(int PLLCON)
{
	
	lcd_sync();
	
volatile unsigned int *LCDPTR=(unsigned int *)LCD_REGS;
// TURN LCD OFF	
LCDPTR[0]&=0xFFFFFFFE;	
	
	if(PLLCON>0xffffff) {

	// SWITCH TO SLOW MODE
	*HWREG(CLK_REGS,0x10)=(*HWREG(CLK_REGS,0x10)& (~0x37)) | (PLLCON>>24);
	*HWREG(CLK_REGS,0x14)=*HWREG(CLK_REGS,0x14)& (~3);	// SET HCLK=FCLK; PCLK=HCLK=FCLK;

	// SWITCH MEMORY TIMINGS TO NORMAL
	*HWREG(MEM_REGS,0x4)= (*HWREG(MEM_REGS,0x4)&(~0x700)) | 0x300;
	*HWREG(MEM_REGS,0x8)=(*HWREG(MEM_REGS,0x8)&(~0x700)) | 0x300;

	// FIX TIMERS OPERATION
	__tmr_fix();	
	
	// FIX LCD OPERATION
	__lcd_fix();
	
	
	return ((PLLCON>>24)&7)? 6000000/((PLLCON>>24)&7) : 12000000;
	}
	
	// SWITCH TO FAST MODE
	int m=((PLLCON>>12)&0xff)+8,p=((PLLCON>>4)&0x1f)+2,s=PLLCON&3;
	
	int FCLK=((unsigned)m*12000000)/(p*(1<<s));

	// SET MEMORY TIMINGS FOR VERY FAST MODES
	if(FCLK>75000000) {
	*HWREG(MEM_REGS,0x4)= (*HWREG(MEM_REGS,0x4)&(~0x700)) | 0x700;
	*HWREG(MEM_REGS,0x8)=(*HWREG(MEM_REGS,0x8)&(~0x700)) | 0x700;
	}
	
	// TURN PLL ON, ENTER SLOW MODE
	//*HWREG(CLK_REGS,0x10)=(*HWREG(CLK_REGS,0x10)& (~0x37)) | 0x10;
	
	// CHANGE SPEED
	*HWREG(CLK_REGS,0x4)=PLLCON;
	
	// KEEP HCLK AROUND 37.5 MHZ
	*HWREG(CLK_REGS,0x14)|=3;	// SET HCLK=FCLK/2; PCLK=HCLK/2;
	
	// SET TIMING FOR SLOWER MODES
	if(FCLK<=75000000) {
	*HWREG(MEM_REGS,0x4)= (*HWREG(MEM_REGS,0x4)&(~0x700)) | 0x300;
	*HWREG(MEM_REGS,0x8)=(*HWREG(MEM_REGS,0x8)&(~0x700)) | 0x300;
	}

	// DO SOMETHING TO WASTE PLL LOCK TIME	

	// ENTER FAST MODE AFTER PLL LOCK TIME
	*HWREG(CLK_REGS,0x10)=(*HWREG(CLK_REGS,0x10)& (~0x37));

	
	
	// FIX LCD TIMING	
	__lcd_fix();
	
	// FIX TIMERS
	__tmr_fix();
	
return FCLK;	
}



// USER-FRIENDLY VERSION, ENSURE PROPER PLLCON IS USED
int cpu_setspeed(int mhz)
{
if(mhz>=192000000) return __cpu_setspeed(CLK_192MHZ);
if(mhz>=152000000) return __cpu_setspeed(CLK_152MHZ);
if(mhz>=120000000) return __cpu_setspeed(CLK_120MHZ);
if(mhz>= 75000000) return __cpu_setspeed(CLK_75MHZ);
if(mhz>= 48000000) return __cpu_setspeed(CLK_48MHZ);
if(mhz>= 12000000) return __cpu_setspeed(CLK_12MHZ);
//if(mhz>=  6000000) return __cpu_setspeed(CLK_6MHZ);
return __cpu_setspeed(CLK_6MHZ);
}


