//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***



// $Id: qsort.c 514 2007-11-18 03:11:26Z ingo $

// ROMLIB CHECKED
// < ibl ; 2006-04-06 >

#include <stdlib.h>

static    void qsort1(char *, char *, size_t);
static    int (*qcompar)(const char *, const char *);
static    void qexchg(char *, char *, size_t);
static    void q3exchg(char *, char *, char *, size_t);

void
qsort(void *base, size_t n_elements, size_t width,
      int (*compar)(const void *, const void *))
{
    
    if (!n_elements) return;
    qcompar = (int (*)(const char *, const char *)) compar;
    qsort1(base, (char *)base + (n_elements - 1) * width, width);
}

static void
qsort1(char *a1, char *a2, register size_t width)
{
    register char *left, *right;
    register char *lefteq, *righteq;
    int cmp;

    for (;;) {
        if (a2 <= a1) return;
        left = a1;
        right = a2;
        lefteq = righteq = a1 + width * (((a2-a1)+width)/(2*width));
        
repeat:
        while (left < lefteq && (cmp = (*qcompar)(left, lefteq)) <= 0) 
            if (cmp < 0) 
                left += width;
            else {
                
                lefteq -= width;
                qexchg(left, lefteq, width);
            }
        
        while (right > righteq) {
            if ((cmp = (*qcompar)(right, righteq)) < 0) {
                
                if (left < lefteq) {
                    qexchg(left, right, width);
                    left += width;
                    right -= width;
                    goto repeat;
                }
            
                righteq += width;
                q3exchg(left, righteq, right, width);
                lefteq += width;
                left = lefteq;
            }
            else if (cmp == 0) {
                righteq += width;
                qexchg(right, righteq, width);
            }
            else    right -= width;
        }
        if (left < lefteq) {
            
            lefteq -= width;
            q3exchg(right, lefteq, left, width);
            righteq -= width;
            right = righteq;
            goto repeat;
        }
    
        qsort1(a1, lefteq - width, width);
        a1 = righteq + width;
    }
    
}

static void
qexchg(register char *p, register char *q,
      register size_t n)
{
    register int c;

    while (n-- > 0) {
        c = *p;
        *p++ = *q;
        *q++ = c;
    }
}

static void
q3exchg(register char *p, register char *q, register char *r,
       register size_t n)
{
    register int c;

    while (n-- > 0) {
        c = *p;
        *p++ = *r;
        *r++ = *q;
        *q++ = c;
    }
}
